aboutsummaryrefslogtreecommitdiff
path: root/lib/service
diff options
context:
space:
mode:
authorFraxinas <andreas.frisch@multimedia-labs.de>2010-04-09 08:25:08 +0200
committerFraxinas <andreas.frisch@multimedia-labs.de>2010-04-09 08:25:08 +0200
commit98c7a787df63a93f868548c2b1e357c0d873ebbe (patch)
treee8258f3e175e540e6106e0b7202abcf550b825d1 /lib/service
parentae60e9e3642949a91b7ea4f77374495fec9a51ed (diff)
parent76250cdc36d0f0e84505d5654066229b846f035f (diff)
downloadenigma2-98c7a787df63a93f868548c2b1e357c0d873ebbe.tar.gz
enigma2-98c7a787df63a93f868548c2b1e357c0d873ebbe.zip
Merge branch 'experimental' of git.opendreambox.org:/git/enigma2 into experimental
Diffstat (limited to 'lib/service')
-rw-r--r--lib/service/iservice.h3
-rw-r--r--lib/service/listboxservice.cpp78
-rw-r--r--lib/service/listboxservice.h8
-rw-r--r--lib/service/servicedvb.cpp137
-rw-r--r--lib/service/servicedvb.h8
-rw-r--r--lib/service/servicemp3.cpp10
6 files changed, 197 insertions, 47 deletions
diff --git a/lib/service/iservice.h b/lib/service/iservice.h
index c35313f0..2ba7cb46 100644
--- a/lib/service/iservice.h
+++ b/lib/service/iservice.h
@@ -356,6 +356,8 @@ public:
sTagCRC,
sTagChannelMode,
+ sTransferBPS,
+
sUser = 0x100
};
enum {
@@ -594,6 +596,7 @@ class iTimeshiftService: public iObject
public:
virtual RESULT startTimeshift()=0;
virtual RESULT stopTimeshift(bool swToLive=true)=0;
+ virtual RESULT setNextPlaybackFile(const char *fn)=0; // not needed by our internal timeshift.. but external plugin...
virtual int isTimeshiftActive()=0;
/* this essentially seeks to the relative end of the timeshift buffer */
diff --git a/lib/service/listboxservice.h b/lib/service/listboxservice.h
index 5228a2f2..589afba6 100644
--- a/lib/service/listboxservice.h
+++ b/lib/service/listboxservice.h
@@ -49,6 +49,7 @@ public:
celServiceNumber,
celMarkerPixmap,
celFolderPixmap,
+ celServiceEventProgressbar,
celServiceName,
celServiceTypePixmap,
celServiceInfo, // "now" event
@@ -62,6 +63,7 @@ public:
picServiceGroup,
picFolder,
picMarker,
+ picServiceEventProgressbar,
picElements
};
@@ -84,6 +86,12 @@ public:
markedBackground,
markedBackgroundSelected,
serviceNotAvail,
+ serviceEventProgressbarColor,
+ serviceEventProgressbarColorSelected,
+ serviceEventProgressbarBorderColor,
+ serviceEventProgressbarBorderColorSelected,
+ serviceDescriptionColor,
+ serviceDescriptionColorSelected,
colorElements
};
diff --git a/lib/service/servicedvb.cpp b/lib/service/servicedvb.cpp
index 558bf0c2..e498dd42 100644
--- a/lib/service/servicedvb.cpp
+++ b/lib/service/servicedvb.cpp
@@ -910,7 +910,7 @@ RESULT eServiceFactoryDVB::lookupService(ePtr<eDVBService> &service, const eServ
return 0;
}
-eDVBServicePlay::eDVBServicePlay(const eServiceReference &ref, eDVBService *service):
+eDVBServicePlay::eDVBServicePlay(const eServiceReference &ref, eDVBService *service):
m_reference(ref), m_dvb_service(service), m_have_video_pid(0), m_is_paused(0)
{
m_is_primary = 1;
@@ -1021,12 +1021,13 @@ void eDVBServicePlay::serviceEvent(int event)
updateTimeshiftPids();
if (!m_timeshift_active)
updateDecoder();
- if (m_first_program_info && m_is_pvr)
+ if (m_first_program_info & 1 && m_is_pvr)
{
- m_first_program_info = 0;
+ m_first_program_info &= ~1;
seekTo(0);
}
- m_event((iPlayableService*)this, evUpdatedInfo);
+ if (!m_timeshift_active)
+ m_event((iPlayableService*)this, evUpdatedInfo);
break;
}
case eDVBServicePMTHandler::eventPreStart:
@@ -1046,17 +1047,85 @@ void eDVBServicePlay::serviceEventTimeshift(int event)
switch (event)
{
case eDVBServicePMTHandler::eventNewProgramInfo:
+ eDebug("eventNewProgramInfo TS");
if (m_timeshift_active)
+ {
updateDecoder();
+ if (m_first_program_info & 2)
+ {
+ if (m_slowmotion)
+ {
+ eDebug("re-apply slowmotion after timeshift file change");
+ m_decoder->setSlowMotion(m_slowmotion);
+ }
+ if (m_fastforward)
+ {
+ eDebug("re-apply skip %d, ratio %d after timeshift file change", m_skipmode, m_fastforward);
+ if (m_skipmode)
+ m_cue->setSkipmode(m_skipmode * 90000); /* convert to 90000 per second */
+ if (m_fastforward != 1)
+ m_decoder->setFastForward(m_fastforward);
+ else
+ m_decoder->setTrickmode();
+ }
+ else
+ seekTo(0);
+ m_first_program_info &= ~2;
+ }
+ m_event((iPlayableService*)this, evUpdatedInfo);
+ }
break;
case eDVBServicePMTHandler::eventSOF:
- m_event((iPlayableService*)this, evSOF);
+#if 0
+ if (!m_timeshift_file_next.empty())
+ {
+ eDebug("timeshift SOF, switch to next file");
+ m_decoder->pause();
+
+ m_first_program_info |= 2;
+
+ eServiceReferenceDVB r = (eServiceReferenceDVB&)m_reference;
+ r.path = m_timeshift_file_next;
+
+ /* free the timeshift service handler, we need the resources */
+ m_service_handler_timeshift.free();
+ resetTimeshift(1);
+
+ if (m_skipmode < 0)
+ m_cue->seekTo(0, -1000);
+ m_service_handler_timeshift.tune(r, 1, m_cue, 0, m_dvb_service); /* use the decoder demux for everything */
+
+ m_event((iPlayableService*)this, evUser+1);
+ }
+ else
+#endif
+ m_event((iPlayableService*)this, evSOF);
break;
case eDVBServicePMTHandler::eventEOF:
if ((!m_is_paused) && (m_skipmode >= 0))
{
- eDebug("timeshift EOF, so let's go live");
- switchToLive();
+ if (m_timeshift_file_next.empty())
+ {
+ eDebug("timeshift EOF, so let's go live");
+ switchToLive();
+ }
+ else
+ {
+ eDebug("timeshift EOF, switch to next file");
+
+ m_first_program_info |= 2;
+
+ eServiceReferenceDVB r = (eServiceReferenceDVB&)m_reference;
+ r.path = m_timeshift_file_next;
+
+ /* free the timeshift service handler, we need the resources */
+ m_service_handler_timeshift.free();
+ resetTimeshift(1);
+
+ m_service_handler_timeshift.tune(r, 1, m_cue, 0, m_dvb_service); /* use the decoder demux for everything */
+
+ m_event((iPlayableService*)this, evUser+1);
+ }
}
break;
}
@@ -2240,38 +2309,32 @@ void eDVBServicePlay::updateTimeshiftPids()
}
}
+RESULT eDVBServicePlay::setNextPlaybackFile(const char *f)
+{
+ m_timeshift_file_next = f;
+ return 0;
+}
+
void eDVBServicePlay::switchToLive()
{
if (!m_timeshift_active)
return;
-
+
eDebug("SwitchToLive");
-
- m_cue = 0;
- m_decoder = 0;
- m_decode_demux = 0;
- m_teletext_parser = 0;
- m_rds_decoder = 0;
- m_subtitle_parser = 0;
- m_new_dvb_subtitle_page_connection = 0;
- m_new_subtitle_page_connection = 0;
- m_rds_decoder_event_connection = 0;
- m_video_event_connection = 0;
+
+ resetTimeshift(0);
+
m_is_paused = m_skipmode = m_fastforward = m_slowmotion = 0; /* not supported in live mode */
- /* free the timeshift service handler, we need the resources */
+ /* free the timeshift service handler, we need the resources */
m_service_handler_timeshift.free();
- m_timeshift_active = 0;
- m_timeshift_changed = 1;
updateDecoder(true);
}
-void eDVBServicePlay::switchToTimeshift()
+void eDVBServicePlay::resetTimeshift(int start)
{
- if (m_timeshift_active)
- return;
-
+ m_cue = 0;
m_decode_demux = 0;
m_decoder = 0;
m_teletext_parser = 0;
@@ -2281,14 +2344,28 @@ void eDVBServicePlay::switchToTimeshift()
m_new_dvb_subtitle_page_connection = 0;
m_rds_decoder_event_connection = 0;
m_video_event_connection = 0;
-
- m_timeshift_active = 1;
m_timeshift_changed = 1;
+ m_timeshift_file_next.clear();
+
+ if (start)
+ {
+ m_cue = new eCueSheet();
+ m_timeshift_active = 1;
+ }
+ else
+ m_timeshift_active = 0;
+}
+
+void eDVBServicePlay::switchToTimeshift()
+{
+ if (m_timeshift_active)
+ return;
+
+ resetTimeshift(1);
eServiceReferenceDVB r = (eServiceReferenceDVB&)m_reference;
r.path = m_timeshift_file;
- m_cue = new eCueSheet();
m_cue->seekTo(0, -1000);
m_service_handler_timeshift.tune(r, 1, m_cue, 0, m_dvb_service); /* use the decoder demux for everything */
@@ -2351,8 +2428,6 @@ void eDVBServicePlay::updateDecoder(bool sendSeekableStateChanged)
if (!m_decoder)
{
h.getDecodeDemux(m_decode_demux);
- if (m_timeshift_changed)
- m_decoder = 0;
if (m_decode_demux)
{
m_decode_demux->getMPEGDecoder(m_decoder, m_is_primary);
diff --git a/lib/service/servicedvb.h b/lib/service/servicedvb.h
index 1262836e..dafaf35d 100644
--- a/lib/service/servicedvb.h
+++ b/lib/service/servicedvb.h
@@ -162,6 +162,7 @@ public:
RESULT stopTimeshift(bool swToLive=true);
int isTimeshiftActive();
RESULT activateTimeshift();
+ RESULT setNextPlaybackFile(const char *fn);
// iCueSheet
PyObject *getCutList();
@@ -213,9 +214,8 @@ private:
int m_is_pvr, m_is_paused, m_timeshift_enabled, m_timeshift_active, m_timeshift_changed;
int m_first_program_info;
- std::string m_timeshift_file;
+ std::string m_timeshift_file, m_timeshift_file_next;
int m_timeshift_fd;
-
ePtr<iDVBDemux> m_decode_demux;
int m_current_audio_stream;
@@ -228,8 +228,10 @@ private:
void updateTimeshiftPids();
void switchToLive();
+
+ void resetTimeshift(int start);
void switchToTimeshift();
-
+
void updateDecoder(bool sendSeekableStateChanged=false);
int m_skipmode;
diff --git a/lib/service/servicemp3.cpp b/lib/service/servicemp3.cpp
index 2c84f7be..0b2ac9d4 100644
--- a/lib/service/servicemp3.cpp
+++ b/lib/service/servicemp3.cpp
@@ -46,6 +46,7 @@ eServiceFactoryMP3::eServiceFactoryMP3()
extensions.push_back("mp4");
extensions.push_back("mov");
extensions.push_back("m4a");
+ extensions.push_back("m2ts");
sc->addServiceFactory(eServiceFactoryMP3::id, this, extensions);
}
@@ -302,11 +303,11 @@ eServiceMP3::eServiceMP3(eServiceReference ref)
if ( ret == -1 ) // this is a "REAL" VCD
uri = g_strdup_printf ("vcd://");
else
- uri = g_strdup_printf ("file://%s", filename);
+ uri = g_filename_to_uri(filename, NULL, NULL);
}
else
- uri = g_strdup_printf ("file://%s", filename);
+ uri = g_filename_to_uri(filename, NULL, NULL);
eDebug("eServiceMP3::playbin2 uri=%s", uri);
@@ -340,9 +341,8 @@ eServiceMP3::eServiceMP3(eServiceReference ref)
struct stat buffer;
if (stat(srt_filename, &buffer) == 0)
{
- std::string suburi = "file://" + (std::string)srt_filename;
- eDebug("eServiceMP3::subtitle uri: %s",suburi.c_str());
- g_object_set (G_OBJECT (m_gst_playbin), "suburi", suburi.c_str(), NULL);
+ eDebug("eServiceMP3::subtitle uri: %s", g_filename_to_uri(srt_filename, NULL, NULL));
+ g_object_set (G_OBJECT (m_gst_playbin), "suburi", g_filename_to_uri(srt_filename, NULL, NULL), NULL);
subtitleStream subs;
subs.type = stSRT;
subs.language_code = std::string("und");
diff --git a/lib/service/listboxservice.cpp b/lib/service/listboxservice.cpp
index 05aaf731..cc8c32b5 100644
--- a/lib/service/listboxservice.cpp
+++ b/lib/service/listboxservice.cpp
@@ -526,7 +526,12 @@ void eListboxServiceContent::paint(gPainter &painter, eWindowStyle &style, const
ePtr<iStaticServiceInformation> service_info;
m_service_center->info(*m_cursor, service_info);
eServiceReference ref = *m_cursor;
- bool isPlayable = !(ref.flags & eServiceReference::isDirectory || ref.flags & eServiceReference::isMarker);
+ bool isMarker = ref.flags & eServiceReference::isMarker;
+ bool isPlayable = !(ref.flags & eServiceReference::isDirectory || isMarker);
+ bool paintProgress = false;
+ ePtr<eServiceEvent> evt;
+
+ bool serviceAvail = true;
if (!marked && isPlayable && service_info && m_is_playable_ignore.valid() && !service_info->isPlayable(*m_cursor, m_is_playable_ignore))
{
@@ -534,12 +539,13 @@ void eListboxServiceContent::paint(gPainter &painter, eWindowStyle &style, const
painter.setForegroundColor(m_color[serviceNotAvail]);
else
painter.setForegroundColor(gRGB(0xbbbbbb));
+ serviceAvail = false;
}
if (selected && local_style && local_style->m_selection)
painter.blit(local_style->m_selection, offset, eRect(), gPainter::BT_ALPHATEST);
- int xoffset=0; // used as offset when painting the folder/marker symbol
+ int xoffset=0; // used as offset when painting the folder/marker symbol or the serviceevent progress
for (int e = 0; e < celElements; ++e)
{
@@ -583,13 +589,19 @@ void eListboxServiceContent::paint(gPainter &painter, eWindowStyle &style, const
}
case celServiceInfo:
{
- ePtr<eServiceEvent> evt;
- if ( isPlayable && service_info && !service_info->getEvent(*m_cursor, evt) )
+ if ( isPlayable && !service_info->getEvent(*m_cursor, evt) )
{
std::string name = evt->getEventName();
if (!name.length())
continue;
text = '(' + evt->getEventName() + ')';
+ if (serviceAvail)
+ {
+ if (!selected && m_color_set[serviceDescriptionColor])
+ painter.setForegroundColor(m_color[serviceDescriptionColor]);
+ else if (selected && m_color_set[serviceDescriptionColorSelected])
+ painter.setForegroundColor(m_color[serviceDescriptionColorSelected]);
+ }
}
else
continue;
@@ -608,9 +620,9 @@ void eListboxServiceContent::paint(gPainter &painter, eWindowStyle &style, const
{
eRect bbox = para->getBoundBox();
int name_width = bbox.width()+8;
- m_element_position[celServiceInfo].setLeft(area.left()+name_width);
+ m_element_position[celServiceInfo].setLeft(area.left()+name_width+xoffs);
m_element_position[celServiceInfo].setTop(area.top());
- m_element_position[celServiceInfo].setWidth(area.width()-name_width);
+ m_element_position[celServiceInfo].setWidth(area.width()-(name_width+xoffs));
m_element_position[celServiceInfo].setHeight(area.height());
}
@@ -678,12 +690,62 @@ void eListboxServiceContent::paint(gPainter &painter, eWindowStyle &style, const
painter.clippop();
}
}
+ else if (e == celServiceEventProgressbar)
+ {
+ eRect area = m_element_position[celServiceEventProgressbar];
+ if (area.width() > 0 && (isPlayable || isMarker))
+ {
+ // we schedule it to paint it as last element.. so we dont need to reset fore/background color
+ paintProgress = isPlayable;
+ xoffset = area.width() + 10;
+ }
+ }
}
-
if (selected && (!local_style || !local_style->m_selection))
style.drawFrame(painter, eRect(offset, m_itemsize), eWindowStyle::frameListboxEntry);
+ if (paintProgress && evt)
+ {
+ eRect area = m_element_position[celServiceEventProgressbar];
+ if (!selected && m_color_set[serviceEventProgressbarBorderColor])
+ painter.setForegroundColor(m_color[serviceEventProgressbarBorderColor]);
+ else if (selected && m_color_set[serviceEventProgressbarBorderColorSelected])
+ painter.setForegroundColor(m_color[serviceEventProgressbarBorderColorSelected]);
+
+ int border = 1;
+ int progressH = 6;
+ int progressX = area.left() + offset.x();
+ int progressW = area.width() - 2 * border;
+ int progressT = offset.y() + (m_itemsize.height() - progressH - 2*border) / 2;
+
+ // paint progressbar frame
+ painter.fill(eRect(progressX, progressT, area.width(), border));
+ painter.fill(eRect(progressX, progressT + border, border, progressH));
+ painter.fill(eRect(progressX, progressT + progressH + border, area.width(), border));
+ painter.fill(eRect(progressX + area.width() - border, progressT + border, border, progressH));
+
+ // calculate value
+ time_t now = time(0);
+ int value = progressW * (now - evt->getBeginTime()) / evt->getDuration();
+
+ eRect tmp = eRect(progressX + border, progressT + border, value, progressH);
+ ePtr<gPixmap> &pixmap = m_pixmaps[picServiceEventProgressbar];
+ if (pixmap)
+ {
+ area.moveBy(offset);
+ painter.clip(area);
+ painter.blit(pixmap, ePoint(progressX + border, progressT + border), tmp, gPainter::BT_ALPHATEST);
+ painter.clippop();
+ }
+ else
+ {
+ if (!selected && m_color_set[serviceEventProgressbarColor])
+ painter.setForegroundColor(m_color[serviceEventProgressbarColor]);
+ else if (selected && m_color_set[serviceEventProgressbarColorSelected])
+ painter.setForegroundColor(m_color[serviceEventProgressbarColorSelected]);
+ painter.fill(tmp);
+ }
+ }
}
-
painter.clippop();
}